/* wptHTTP.h - Generic network code for HTTP access
 *	Copyright (C) 2004, 2005, 2006, 2007 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */
#ifndef WPT_HTTP_H
#define WPT_HTTP_H

/* HTTP status codes. */
enum http_statcode_t {
    HTTP_STAT_NONE = 0,
    HTTP_STAT_200 = 200,  /* OK */
    HTTP_STAT_201 = 201,  /* Created */
    HTTP_STAT_301 = 301,  /* Moved Permanently */
    HTTP_STAT_302 = 302,  /* Moved Temporarily */
    HTTP_STAT_400 = 400,  /* Bad Request */
    HTTP_STAT_401 = 401,  /* Unauthorized */
    HTTP_STAT_403 = 403,  /* Forbidden */
    HTTP_STAT_404 = 404,  /* Not Found */
    HTTP_STAT_405 = 405,  /* Method Not Allowed */
};


/* HTTP header list. */
struct http_head_s {
    struct http_head_s *next;
    char d[1];
};
typedef struct http_head_s *http_head_t;

class NetHTTP {
public:
    /* HTTP methods. */
    enum  { HTTP_GET, HTTP_HEAD, HTTP_PUT, HTTP_POST };

private:
    http_head_t req_headers;
    http_head_t resp_headers;
    int fd;
    int statcode;
    char *host;
    int port;
    char *url;
    int nleft;
    int method;
    int ver;
    int error;

private:
    void reset (void);
    int parseResponse (int *statcode);
    int connect (const char *host, int port);
    int isDataAvailable (int fd);
    int readLine (char *buf, unsigned int nbuf, int nonblock, int *nn, int *eof);
    void addHeader (http_head_t *root, const char *val);
    bool findHeader (http_head_t root, 
		     const char *name, const char **val);
    int parseHeaders (http_head_t *r_head);

    int extractHostInfo (const char *url, char **host, char **new_url);
    int sendRequest (const char *host, int port, const char *url);

public:
    NetHTTP ();
    NetHTTP (const char *url);
    NetHTTP (const char *host, int port, const char *url);
    ~NetHTTP ();
    
    void setVersion (int ver);
    int getErrorCode (void);
    int getStatusCode (void);
    unsigned int getContentLength (void);
    const char *getContentType (void);

    int addRequestHeader (const char *name, const char *val);
    int addRequestHeader (const char *name, unsigned int val);
    
    int open (const char *url);
    int write (const void *buf, unsigned int buflen);
    int read (void *buf, unsigned int buflen);
    int readData (FILE *out);

    int get (const char *url);
    int head (const char *url);
};

#endif /* WPT_HTTP_H */
