/* wptGPGOptDlg.cpp - WinPT GnuPG GPG Config
 *	Copyright (C) 2001-2003, 2005, 2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>

#include "wptTypes.h"
#include "wptNLS.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptW32API.h"
#include "wptErrors.h"
#include "resource.h"


/* Dialog box procedure for manipulating the GPG config file. */
BOOL CALLBACK
gpgopt_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    int nbytes;
    char *optbuf;
    
    switch (msg)  {
    case WM_INITDIALOG:
	optbuf = get_gnupg_config ();
	if (!optbuf) {
	    msg_box (dlg,  _("Could not load GnuPG config file!"),
		     _("GPG Config"), MB_ERR);
	    EndDialog (dlg, FALSE);
	    return TRUE;
	}
	SetDlgItemText (dlg, IDC_GPGOPT_FILE, optbuf);
	SetWindowText (dlg, _("GnuPG Configuration File" ));
	SetDlgItemText (dlg, IDC_GPGOPT_SAVE, _("&Save"));
	SetDlgItemText (dlg, IDC_GPGOPT_LOAD, _("&Load"));
	SetDlgItemText (dlg, IDCANCEL, _("&Exit"));
	free_if_alloc (optbuf);
	SetForegroundWindow (dlg);
	return TRUE;
        
    case WM_COMMAND:
        switch (LOWORD (wparam)) {
        case IDC_GPGOPT_SAVE:
	    nbytes = item_get_text_length (dlg, IDC_GPGOPT_FILE);
	    if (nbytes < 1) {
		msg_box (dlg, _("The 'gpg.conf' file is not loaded."), 
			 _("GPG Config"), MB_ERR);
		return TRUE;
	    }
            optbuf = new char[nbytes + 2];
            if (!optbuf)
		BUG (0);
	    memset (optbuf, 0, nbytes + 2);
            nbytes = GetDlgItemText (dlg, IDC_GPGOPT_FILE, optbuf, nbytes+1);
            if (check_gnupg_options (optbuf, 1)) {
                msg_box (dlg, _("Config contains invalid GnuPG keywords."),
			 _("GPG Config"), MB_ERR);
                free_if_alloc (optbuf);
                return TRUE;
            }
            if (set_gnupg_options (optbuf, strlen (optbuf))) {
                msg_box (dlg, _("Could not save GnuPG config file."), 
			 _("GPG Config"), MB_ERR);
                free_if_alloc (optbuf);
                return TRUE;
            }
	    /* We need to call gpg after the gpg.conf is stored, otherwise
	       it would still check the old content. */
	    if (!gpg_check_return_code ("--gpgconf-list"))
		msg_box (dlg, _("Successfully saved."), _("GPG Config"), MB_OK);
	    else
		msg_box (dlg, _("GnuPG returned an error while parsing the config file."),
			 _("GPG Config"), MB_ERR);
            free_if_alloc (optbuf);
            return TRUE;
            
        case IDC_GPGOPT_LOAD:
	    if (item_get_text_length (dlg, IDC_GPGOPT_FILE) > 2) {
                if (msg_box (dlg, _("Current data will be lost!\n"
                                    "Are you sure?"), _("GPG Config"), 
				    MB_WARN|MB_YESNO) == IDNO)
                    return TRUE;
            }
            optbuf = get_gnupg_config ();
            if (!optbuf)
                msg_box (dlg, _("Could not load GnuPG config file."), 
			 _("GPG Config"), MB_ERR);
	    else {
		SetDlgItemText (dlg, IDC_GPGOPT_FILE, optbuf);
		free_if_alloc (optbuf);
	    }
            return TRUE;
            
        case IDCANCEL:
            EndDialog (dlg, FALSE);
            return TRUE;
        }
        break;
    }
    
    return 0;
}
